#!/bin/ksh
# ******************************************************************************************************************************
# * Licensed Materials - Property of IBM                                                                                       *
# * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
# *                                                                                                                            *
# *US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
# ******************************************************************************************************************************

rcrdone(){
echo $MESSAGE:$EXITCODE
echo $CRREWECH002$
exit $EXITCODE
}

chmod -R +x lang

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "en" ]
then 
     LOCALE="en"
     SQLVARF="utf8/vars_en.sql"
	. lang/en/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "it" ]
then 
	LOCALE="it"
	SQLVARF="utf8/vars_it.sql"
	. lang/it/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "pl" ]
then 
	LOCALE="pl"
	SQLVARF="utf8/vars_pl.sql"
	. lang/pl/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "ko" ]
then 
	LOCALE="kr"
	SQLVARF="utf8/vars_kr.sql"
	. lang/kr/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "ja" ]
then 
	LOCALE="ja"
	SQLVARF="utf8/vars_ja.sql"
	. lang/ja/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*_[A-Z][A-Z]\).*/\1/'` == "zh_CN" ]
then 
	LOCALE="zh_CN"
	SQLVARF="utf8/vars_zh_CN.sql"
	. lang/zh_CN/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*_[A-Z][A-Z]\).*/\1/'` == "zh_TW" ]
then 
	LOCALE="zh_TW"
	SQLVARF="utf8/vars_zh_TW.sql"
	. lang/zh_TW/messages.sh
fi

if [ -z $LOCALE ]
then
	 LOCALE="en"
	 SQLVARF="utf8/vars_en.sql"
	. lang/en/messages.sh
fi	


SILENT=0
MESSAGE="No errors"
EXITCODE=0

SILENT=0
ORA_CMD="OFF"

if [ $# -eq 1 ]
	then 
	if [ $1 == "-silent" ]
	then
			SILENT=1
	else 
			SILENT=0
	fi
	if [ $1 == "-debug" ]
	then
		ORA_CMD="ON"
		else 
			ORA_CMD="OFF"
	fi
fi

if [ $# -eq 2 ]
then
	if [ $2 -eq "-debug" ]
	 then
			ORA_CMD="ON"
	else 
			ORA_CMD="OFF"
	fi
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWECH033
fi	

if [ $SILENT -eq 0 ]
then
	if [ -z $ORACLE_HOME ]
	then 
		echo $CRREWPRMPT013
		read ORACLE_HOME
	fi
fi

if [ -z $ORACLE_HOME ]
then
	MESSAGE=$CRREWMSGE030
	EXITCODE=$CRREW4530E
	rcrdone
fi

if [ ! -d $ORACLE_HOME/bin ]
then
	MESSAGE=$CRREWMSGE031
	EXITCODE=$CRREW4531E
	rcrdone
fi	

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT028
	read ORACLE_SID
fi

if [ -z $ORACLE_SID ]
then
	MESSAGE=$CRREWMSGE034
	EXITCODE=$CRREW4534E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT016
	read SYSPWD
fi

if [ -z $SYSPWD ]
then
	MESSAGE=$CRREWMSGE035
	EXITCODE=$CRREW4535E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT003
	read DBA_USER
fi

if [ -z $DBA_USER ]
then
	MESSAGE=$CRREWMSGE039
	EXITCODE=$CRREW4539E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT021
	read DBA_PASSWORD
fi

if [ -z $DBA_PASSWORD ]
then
	MESSAGE=$CRREWMSGE040
	EXITCODE=$CRREW4540E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT029
	read NF_SCHEMA
fi

if [ -z $NF_SCHEMA ]
then
	MESSAGE=$CRREWMSGE005
	EXITCODE=$CRREW4505E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT030
	read STAR_SCHEMA
fi

if [ -z $STAR_SCHEMA ]
then
	MESSAGE=$CRREWMSGE006
	EXITCODE=$CRREW4506E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT031
	read REP_USER
fi

if [ -z $REP_USER ]
then
	MESSAGE=$CRREWMSGE007
	EXITCODE=$CRREW4507E
	rcrdone
fi

if [ $DBA_USER == $REP_USER ]
then
	EXITCODE=$CRREW4595E
	MESSAGE=$CRREWMSGE095
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT032
	read VIEW_REDEF
fi

if [ -z $VIEW_REDEF ]
then
    VIEW_REDEF=0
fi

if [ $VIEW_REDEF -ne 1 ]
then
    VIEW_REDEF=0
fi 

CD=`pwd`

if [ ! -d $CD/work_spc ]
then
	mkdir -p $CD/work_spc
else
	rm -fr $CD/work_spc
	mkdir -p $CD/work_spc
fi	
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE076
	EXITCODE=$CRREW4576E
	rcrdone
fi

if [ ! -d $CD/logs ]
then
	mkdir -p $CD/logs
else
	rm -fr $CD/logs
	mkdir -p $CD/logs	
fi
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE076
	EXITCODE=$CRREW4576E
	rcrdone
fi

cp -r ./ddl $CD/work_spc
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE024
	EXITCODE=$CRREW4524E
	rcrdone
fi

cp -r ./scripts $CD/work_spc
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE024
	EXITCODE=$CRREW4524E
	rcrdone
fi

cp ./migrateDW.sql $CD/work_spc
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE024
	EXITCODE=$CRREW4524E
	rcrdone
fi
echo "PROMPT">prev_mig.sql
export ORACLE_SID CD SYSPWD DBA_USER DBA_PASSWORD NF_SCHEMA STAR_SCHEMA REP_USER  ORA_CMD SQLVARF
$ORACLE_HOME/bin/sqlplus /nolog @$CD/work_spc/migrateDW.sql $ORACLE_SID $CD $SYSPWD $DBA_USER $DBA_PASSWORD $NF_SCHEMA $STAR_SCHEMA $REP_USER  $ORA_CMD $SQLVARF $VIEW_REDEF

rcrdone