--******************************************************************************************************************************
--* Licensed Materials - Property of IBM                                                                                       *
--* (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
--*                                                                                                                            *
--*US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
--******************************************************************************************************************************

ALTER SESSION SET CURRENT_SCHEMA=&&DBA_USER;


CREATE OR REPLACE FUNCTION REPLACE_SCHEMA (STATEMENT CHAR, ODS_SCHEMA CHAR,DM_SCHEMA CHAR,REP_USER CHAR)
RETURN VARCHAR
IS
		v_text 									VARCHAR(8192);
		v_ods_schema 							VARCHAR(20);
		v_dm_schema 							VARCHAR(20);
		v_rep_user								VARCHAR(20);
BEGIN		
		v_text:=STATEMENT;
		v_ods_schema:=ODS_SCHEMA|| '.';
		v_dm_schema:=DM_SCHEMA || '.';
		v_rep_user:=REP_USER;
		
		v_text:=COALESCE(REPLACE(v_text,'RIDW.',v_dm_schema),v_text);
		v_text:=COALESCE(REPLACE(v_text,'RIODS.',v_ods_schema),v_text);
		v_text:=COALESCE(REPLACE(v_text,'REPUSER',v_rep_user),v_text);
		RETURN v_text;
END;
/

CREATE OR REPLACE PROCEDURE PROCESS_UOW (MODULE_ID INTEGER,SUB_MODULE_ID INTEGER,UOW_ID INTEGER,ODS_SCHEMA CHAR,DM_SCHEMA CHAR, TARGET_VERSION CHAR,REP_USER CHAR)
AUTHID CURRENT_USER IS

		MESSAGE_TEXT VARCHAR(100);
		RETVAL INTEGER := 0; 
		v_sqlstate CHAR(5) := '00000';
    	v_sqlcode INTEGER := 0;
    	v_errmsg VARCHAR(100) := '';
    	err_stmt VARCHAR(1000) := '';
    	
    	v_module_id 		INTEGER;
		v_sub_module_id 	INTEGER;
      	v_uow_id 			INTEGER;
      	v_ods_schema VARCHAR(20); 
		v_dm_schema VARCHAR(20);
			
		stmt VARCHAR(1000) := ''; 
		v_statement_number 	INTEGER;
		v_area 				VARCHAR(255);
		v_type 				VARCHAR(255);
		v_action 			VARCHAR(255);
		v_data_movement 	INTEGER;
		v_table_refactoring INTEGER;
		v_description 		VARCHAR(1000);
		v_affected_table 	VARCHAR(255);
		v_affected_schema 	VARCHAR(255);
		v_object_type 		VARCHAR(255);
		v_object_name 		VARCHAR(255);
		v_pre_exists 		INTEGER;
		v_recoverable 		INTEGER;
		v_has_dependencies 	INTEGER;
		v_execution_status 	INTEGER;
		v_execute 			INTEGER;
		v_release 			VARCHAR(255);
		v_special_release 	VARCHAR(255);
		v_text 				VARCHAR(8192);
		v_temp_text 		VARCHAR(8192);
		v_failled_pred integer := 0;
		v_target_version VARCHAR(255);
		v_rep_user VARCHAR(20);
		v_schema VARCHAR(255);
		v_current_schema varchar(10);
		TYPE RI_CURSOR IS REF CURSOR;
		s4 RI_CURSOR;
		failed_stmt EXCEPTION;
		failled_dep EXCEPTION;
								


	CURSOR C4 IS 
				SELECT STATEMENT_NUMBER,AREA,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,TEXT
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=v_module_id AND SUB_MODULE_ID=v_sub_module_id AND UOW_ID=v_uow_id
				AND EXECUTE=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version AND PRE_EXISTS=0
				ORDER BY STATEMENT_NUMBER,UOW_ID;
						
			BEGIN
			
			
			v_module_id:=MODULE_ID;
			v_sub_module_id:=SUB_MODULE_ID;
			v_uow_id := UOW_ID;
			v_ods_schema:= ODS_SCHEMA;
			v_dm_schema := DM_SCHEMA;
			v_target_version:= TARGET_VERSION;
			v_rep_user:=REP_USER;
	
				OPEN C4;
				LOOP
				FETCH C4 INTO v_statement_number,v_area,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_text;
				EXIT WHEN C4%NOTFOUND;
				  IF (v_has_dependencies=1) THEN
				  SELECT COUNT(*) INTO v_failled_pred FROM CONFIG.STATEMENT_DEPENDENCIES WHERE STATEMENT_NUMBER=v_statement_number AND EXISTS(SELECT STATEMENT_NUMBER FROM CONFIG.STATEMENT_LIST WHERE 
				  STATEMENT_NUMBER=PREDECESSOR_NUMBER AND (EXECUTION_STATUS <>1 OR EXECUTION_STATUS IS NULL)AND PRE_EXISTS<>1);
					  IF(v_failled_pred>0) THEN
						RAISE failled_dep;
					  END IF;
				  END IF;
				  
					v_temp_text:=REPLACE_SCHEMA(v_text, v_ods_schema,v_dm_schema,v_rep_user);
					 EXECUTE IMMEDIATE v_temp_text;
					IF (v_sqlcode<>0) THEN
  						RAISE failed_stmt;	
				  ELSE
				  	BEGIN
				  		UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=1, RET_CODE='00000' WHERE STATEMENT_NUMBER=v_statement_number;
				  		COMMIT;
						END;
					END IF;
				END LOOP;
				CLOSE C4;
				

		 EXCEPTION 
		 	WHEN failled_dep THEN 
		 	BEGIN
			 	 ROLLBACK;  
		     	 UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE='-20011' WHERE STATEMENT_NUMBER=v_statement_number;
		      	 COMMIT; 
			 	 RAISE_APPLICATION_ERROR(-20011,VARS_LANG.CRREWMSGE102);
		 	END;
		  WHEN failed_stmt THEN
			   BEGIN
			    v_sqlcode:= SQLCODE;
				  ROLLBACK; 
				  UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE=TO_CHAR(v_sqlcode) WHERE STATEMENT_NUMBER=v_statement_number;
			      COMMIT; 
			      RAISE_APPLICATION_ERROR(-20010,VARS_LANG.CRREWMSGE101);
			   END;   
		  WHEN OTHERS THEN
   			BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
		     	UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE=v_sqlcode WHERE STATEMENT_NUMBER=v_statement_number;
		      COMMIT; 
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;
		 RETURN;
      
END;
/

CREATE OR REPLACE PROCEDURE VALIDATE_EXISTENCE(STATEMENT_NUMBER INTEGER, TYPE CHAR,ACTION CHAR,AFFECTED_TABLE CHAR,AFFECTED_SCHEMA CHAR,OBJECT_TYPE CHAR,OBJECT_NAME CHAR,ODS_SCHEMA CHAR,DM_SCHEMA CHAR)
AUTHID CURRENT_USER IS  

			MESSAGE_TEXT VARCHAR(100);
			RETVAL INTEGER := 0; 
			v_sqlstate CHAR(5) := '00000';
    	 	v_sqlcode INTEGER := 0;
    	 	v_errmsg VARCHAR(100) := '';
    	 	err_stmt VARCHAR(1000) := '';
    	 	v_exists INTEGER := 0;
    	 	v_target_version VARCHAR(10);
    	 	v_module INTEGER;
			v_sub_module INTEGER;
			v_ods_schema VARCHAR(20); 
			v_dm_schema VARCHAR(20);
			stmt VARCHAR(1000) := ''; 
			v_statement_number 				INTEGER;
			v_module_id 							INTEGER;
			v_sub_module_id 					INTEGER;
			v_area 										VARCHAR(255);
			v_type 										VARCHAR(255);
			v_action 									VARCHAR(255);
			v_uow_id 									INTEGER;
			v_data_movement 					INTEGER;
			v_table_refactoring 			INTEGER;
			v_description 						VARCHAR(1000);
			v_affected_table 					VARCHAR(255);
			v_affected_schema 				VARCHAR(255);
			v_object_type 						VARCHAR(255);
			v_object_name 						VARCHAR(255);
			v_pre_exists 							INTEGER;
			v_recoverable 						INTEGER;
			v_has_dependencies 				INTEGER;
			v_execution_status 				INTEGER;
			v_execute 								INTEGER;
			v_release 								VARCHAR(255);
			v_special_release 				VARCHAR(255);
			v_text 										VARCHAR(8192);
			any_exception EXCEPTION;
BEGIN    
    	
    	v_action:=ACTION;
    	
    	v_affected_schema:= CASE WHEN AFFECTED_SCHEMA='RIODS' THEN ODS_SCHEMA ELSE DM_SCHEMA END;
    	v_affected_table:=AFFECTED_TABLE;
    	v_object_name:=OBJECT_NAME;
    	v_type:=TYPE;    	
    	v_statement_number:=STATEMENT_NUMBER;
    	    	
IF (v_action='ADD COLUMN') THEN
	     
	     SELECT CASE WHEN EXISTS (SELECT COLUMN_NAME FROM ALL_TAB_COLS WHERE RTRIM(TABLE_NAME)=RTRIM(v_affected_table) AND RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(COLUMN_NAME)=RTRIM(v_object_name) AND RTRIM(DATA_TYPE)=RTRIM(v_type)) THEN 1 ELSE 0 END  INTO v_exists FROM DUAL;
	       	IF (v_exists=0) THEN
	       		 SELECT CASE WHEN EXISTS (SELECT COLUMN_NAME FROM ALL_TAB_COLS WHERE RTRIM(TABLE_NAME)=RTRIM(v_affected_table) AND RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(COLUMN_NAME)=RTRIM(v_object_name)) THEN 1 ELSE 0 END  INTO v_exists FROM DUAL;
	       	ELSE
	       		v_exists:=-1;	
	       	END IF;

	   ELSIF (v_action='CREATE CONSTRAINT') THEN
	       
	        IF (v_type='PRIMARY') THEN
	      			SELECT CASE WHEN EXISTS(SELECT CONSTRAINT_NAME FROM ALL_CONSTRAINTS WHERE RTRIM(CONSTRAINT_NAME)= RTRIM(v_object_name) AND RTRIM(CONSTRAINT_TYPE)='P' AND RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(TABLE_NAME)= RTRIM(v_affected_table)) THEN 1 ELSE 0 END INTO v_exists  FROM DUAL;
	      	ELSE
	      			SELECT CASE WHEN EXISTS(SELECT CONSTRAINT_NAME  FROM ALL_CONSTRAINTS WHERE RTRIM(CONSTRAINT_NAME)= RTRIM(v_object_name) AND RTRIM(CONSTRAINT_TYPE)='R' AND RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(TABLE_NAME)= RTRIM(v_affected_table)) THEN 1 ELSE 0 END INTO v_exists FROM DUAL;
	      	END IF;
	  
	     ELSIF(v_action='CREATE INDEX') THEN	
	      		SELECT CASE WHEN EXISTS(SELECT INDEX_NAME FROM ALL_INDEXES WHERE RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(INDEX_NAME)= RTRIM(v_object_name) ) THEN 1 ELSE 0 END INTO v_exists FROM DUAL;
	     ELSIF (v_action='CREATE TABLE') THEN
	            SELECT CASE WHEN EXISTS(SELECT TABLE_NAME FROM ALL_TABLES WHERE RTRIM(OWNER)=RTRIM(v_affected_schema) AND RTRIM(TABLE_NAME) = RTRIM(v_object_name)) THEN 1 ELSE 0 END INTO v_exists FROM DUAL;
	     ELSIF (v_action='CREATE TRIGGER') THEN
	      	  SELECT CASE WHEN EXISTS(SELECT TRIGGER_NAME  FROM ALL_TRIGGERS WHERE RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(TRIGGER_NAME) = RTRIM(v_object_name)) THEN 1 ELSE 0 END INTO v_exists FROM DUAL;
	     ELSIF (v_action='CREATE VIEW') THEN
	      	  SELECT CASE WHEN EXISTS(SELECT VIEW_NAME  FROM ALL_VIEWS  WHERE RTRIM(OWNER)= RTRIM(v_affected_schema) AND RTRIM(VIEW_NAME) = RTRIM(v_object_name)) THEN 1 ELSE 0 END INTO v_exists FROM DUAL;
	     ELSE v_exists:=0;
 END IF; 
     UPDATE CONFIG.STATEMENT_LIST SET PRE_EXISTS=CASE WHEN v_exists=-1 THEN 2 ELSE v_exists END WHERE STATEMENT_NUMBER=v_statement_number;
     COMMIT;
     UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=CASE WHEN PRE_EXISTS>0 THEN 2 ELSE EXECUTION_STATUS END WHERE STATEMENT_NUMBER=v_statement_number;
      COMMIT;
      
EXCEPTION  
WHEN any_exception THEN
BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
   				DBMS_OUTPUT.put_line(DBMS_UTILITY.format_error_backtrace);
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;  
   			    
WHEN OTHERS THEN
   			BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;      
      RETURN;  
END;
/


CREATE OR REPLACE PROCEDURE MIGRATE_MODULE(MODULE_ID INTEGER,SUB_MODULE_ID INTEGER,ODS_SCHEMA CHAR,DM_SCHEMA CHAR,TARGET_VERSION CHAR,REP_USER CHAR)
AUTHID CURRENT_USER IS

		MESSAGE_TEXT VARCHAR(100);
		RETVAL INTEGER := 0;
		v_sqlstate CHAR(5) := '00000';
    	v_sqlcode INTEGER := 0;
    	v_errmsg VARCHAR(100) := '';
    	err_stmt VARCHAR(1000) := '';

    	 v_target_version VARCHAR(10);
    	 v_target_version2 VARCHAR(10);
    	 v_module INTEGER;
		 v_sub_module INTEGER;
		 v_ods_schema VARCHAR(20);
		 v_dm_schema VARCHAR(20);
		 stmt VARCHAR(1000) := '';
		 v_statement_number 				INTEGER;
		 v_module_id 							INTEGER;
		 v_sub_module_id 					INTEGER;
		 v_module_id2 							INTEGER;
		 v_sub_module_id2 					INTEGER;
		 v_area 										VARCHAR(255);
		 v_type 										VARCHAR(255);
		 v_action 									VARCHAR(255);
		 v_uow_id 									INTEGER;
		 v_data_movement 					INTEGER;
		 v_table_refactoring 			INTEGER;
		 v_description 						VARCHAR(1000);
		 v_affected_table 					VARCHAR(255);
		 v_affected_schema 				VARCHAR(255);
		 v_object_type 						VARCHAR(255);
		 v_object_name 						VARCHAR(255);
		 v_pre_exists 							INTEGER;
		 v_recoverable 						INTEGER;
		 v_has_dependencies 				INTEGER;
		 v_execution_status 				INTEGER;
		 v_execute 								INTEGER;
		 v_release 								VARCHAR(255);
		 v_special_release 				VARCHAR(255);
		 v_text 										VARCHAR(8192);
		 sp_stmt                   VARCHAR(1000) := '';
		 fetch_code CHAR(5) := '00000';
		 v_rep_user VARCHAR(20);
		 any_exception EXCEPTION;
		 CURSOR C2 IS
				SELECT STATEMENT_NUMBER,MODULE_ID,SUB_MODULE_ID,AREA,TYPE,ACTION,UOW_ID,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,EXECUTION_STATUS,"EXECUTE",RELEASE,SPECIAL_RELEASE,TEXT
				FROM CONFIG.STATEMENT_LIST
				WHERE MODULE_ID=v_module AND SUB_MODULE_ID=v_sub_module
				AND "EXECUTE"=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version
				ORDER BY STATEMENT_NUMBER,UOW_ID;

				CURSOR C3  IS
			    SELECT DISTINCT MODULE_ID,SUB_MODULE_ID,UOW_ID
				FROM CONFIG.STATEMENT_LIST
				WHERE MODULE_ID=v_module AND SUB_MODULE_ID=v_sub_module
				AND "EXECUTE"=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version AND PRE_EXISTS=0
				GROUP BY MODULE_ID,SUB_MODULE_ID,UOW_ID
				ORDER BY UOW_ID,MODULE_ID,SUB_MODULE_ID;

BEGIN
			v_module:=MODULE_ID;
			v_sub_module:=SUB_MODULE_ID;
			v_target_version:=TARGET_VERSION;
			v_ods_schema:=ODS_SCHEMA;
			v_dm_schema:=DM_SCHEMA;
			v_rep_user:=REP_USER;



				OPEN C2;
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
				LOOP
				FETCH C2 INTO v_statement_number,	v_module_id2,v_sub_module_id2,v_area,v_type,v_action,v_uow_id,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_execution_status,v_execute,v_release,v_special_release,v_text;
				EXIT WHEN C2%NOTFOUND;
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
				commit;
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
	        	VALIDATE_EXISTENCE(v_statement_number,v_type,v_action,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_ods_schema,v_dm_schema);
	        	COMMIT;
	        	v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
 				END LOOP;
				CLOSE C2;
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
                
                COMMIT;
                v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
				OPEN C3;
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
				LOOP
				FETCH C3 INTO v_module_id2,v_sub_module_id2,v_uow_id;
				EXIT WHEN C3%NOTFOUND;
						--sp_stmt:='EXECUTE CONFIG.PROCESS_UOW(' || COALESCE(TO_CHAR(v_module_id),'') || ',' || COALESCE(TO_CHAR(v_sub_module_id),'') || ',' || COALESCE(TO_CHAR(v_uow_id),'') || ',''' || COALESCE(v_ods_schema,'')|| ''',''' || COALESCE(v_dm_schema,'') || ''',''' || COALESCE(v_target_version,'') || ''',''' || COALESCE(v_rep_user,'') || ''')';
						--EXECUTE IMMEDIATE sp_stmt;
				PROCESS_UOW(v_module_id2,v_sub_module_id2,v_uow_id,v_ods_schema,v_dm_schema,v_target_version,v_rep_user);
				v_sqlcode:=SQLCODE;
				IF v_sqlcode<>0 THEN
					RAISE any_exception;
				END IF;
 				END LOOP;
    		CLOSE C3;
EXCEPTION
WHEN any_exception THEN
BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
   				DBMS_OUTPUT.put_line(DBMS_UTILITY.format_error_backtrace);
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;  
   			
WHEN OTHERS THEN
   			BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
   				DBMS_OUTPUT.put_line(DBMS_UTILITY.format_error_backtrace);
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;    		
    	RETURN;
END;
/    	

CREATE OR REPLACE PROCEDURE DW_MIGRATE(ODS_SCHEMA CHAR,DM_SCHEMA CHAR,REP_USER CHAR)
AUTHID CURRENT_USER IS

 			 MESSAGE_TEXT VARCHAR(100);
			 RETVAL INTEGER := 0; 
			 v_sqlcode INTEGER :=0;
    	     v_errmsg VARCHAR(100) := '';
    	     err_stmt VARCHAR(1000) := '';
    	
    	
			 v_target_version VARCHAR(10) := '1.0.0.2';
			 v_current_version VARCHAR(10);
			 v_migration_version VARCHAR(10);
			 v_module INTEGER;
			 v_sub_module INTEGER;
			 v_ods_schema VARCHAR(20); 
			 v_dm_schema VARCHAR(20);
			 stmt VARCHAR(1000) := ''; 
			 sp_stmt VARCHAR(1000) := '';
			 fetch_code CHAR(5) := '00000';
			 v_rep_user VARCHAR(20);
			 no_dm_schema EXCEPTION;
			 no_ods_schema  EXCEPTION;
			 no_rep_user  EXCEPTION;
			 null_version EXCEPTION;
			 null_target EXCEPTION;
			 current_grt_target EXCEPTION;

			CURSOR CVer IS 		-- the versions to need to be migrated 
				SELECT DISTINCT S.RELEASE,S.MODULE_ID, S.SUB_MODULE_ID FROM CONFIG.STATEMENT_LIST S WHERE S.RELEASE >= v_current_version and S.RELEASE <= v_target_version 
				AND EXISTS(SELECT M.MODULE_ID, M.SUB_MODULE_ID FROM CONFIG.MODULES M WHERE M."MODIFY"='Y' AND M.MODULE_ID=S.MODULE_ID AND M.SUB_MODULE_ID=S.SUB_MODULE_ID)
				ORDER BY S.RELEASE, S.MODULE_ID, S.SUB_MODULE_ID;
			

 BEGIN			
			v_ods_schema:=ODS_SCHEMA;
			v_dm_schema:=DM_SCHEMA;
			v_rep_user:=REP_USER;
			
			SELECT CHAR_VALUE INTO v_current_version FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=6;
			
			SELECT CHAR_VALUE INTO v_target_version FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;
			
			IF (v_current_version IS NULL) THEN
						RAISE null_version;
			END IF;

			IF (v_target_version IS NULL) THEN
						RAISE null_target;
			END IF;
			
      IF (v_current_version > v_target_version) THEN
						RAISE current_grt_target;
			END IF;
			
			IF (v_ods_schema IS NULL) THEN
						RAISE no_ods_schema;
			END IF;
			
			
			IF (v_dm_schema IS NULL) THEN
						RAISE no_dm_schema;
			END IF;
			
			IF (v_rep_user IS NULL) THEN
						RAISE no_rep_user;
			END IF;
			
			OPEN CVer;
			FETCH CVer INTO v_migration_version,v_module,v_sub_module;
				LOOP EXIT WHEN CVer%NOTFOUND;					
					MIGRATE_MODULE(v_module,v_sub_module,v_ods_schema ,v_dm_schema,v_migration_version,v_rep_user);
					FETCH CVer INTO v_migration_version,v_module,v_sub_module;
			END LOOP; 
			CLOSE CVer;

  		COMMIT;		
  		
  		UPDATE CONFIG.SYSCONFIG_INFO SET CHAR_VALUE=v_target_version WHERE INFO_ID=6;
  		--DELETE FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;

 EXCEPTION
	WHEN no_dm_schema THEN
		RAISE_APPLICATION_ERROR(-20004,VARS_LANG.CRREWMSGE006);
	WHEN  no_ods_schema THEN
		RAISE_APPLICATION_ERROR(-20003,VARS_LANG.CRREWMSGE005);
	WHEN no_rep_user THEN
		RAISE_APPLICATION_ERROR(-20006,VARS_LANG.CRREWMSGE007);
	WHEN null_version THEN
		RAISE_APPLICATION_ERROR(-20001,VARS_LANG.CRREWMSGE098);
	WHEN null_target THEN
		RAISE_APPLICATION_ERROR(-20005,VARS_LANG.CRREWMSGE099);
	WHEN current_grt_target THEN
		RAISE_APPLICATION_ERROR(-20002,VARS_LANG.CRREWMSGE100);
	WHEN OTHERS THEN
   			BEGIN
   			  v_sqlcode:= SQLCODE;
   				ROLLBACK;  
		      	RAISE_APPLICATION_ERROR(-20012,'Exception has occured SQLCODE: ' || TO_CHAR(v_sqlcode) || ' SQLERR: ' || SQLERRM);
   			END;
	RETURN;
END;
/  

